/*
 *  ALTerator - ALT Linux configuration project
 *
 *  Copyright (c) 2004,2005 ALT Linux Ltd.
 *  Copyright (c) 2004,2005 Stanislav Ievlev
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *  USA.
 */
#ifndef QTLOOK_DIALOG_HH
#define QTLOOK_DIALOG_HH

#include <iostream>
#include <map>
#include <stack>

#include <qlayout.h>
#include <qwidget.h>
#include <qapplication.h>
#include <qeventloop.h>

#include <eval.hh>
#include <dialog.hh>

namespace qtlook
{
    class dialog_widget: public QWidget
    {
	Q_OBJECT
	public:
	    dialog_widget();
	    std::string lastaction_;
	    void exec();
	    void closeEvent (QCloseEvent * e);
	protected slots:
	    void intercept(const std::string& id);
	    void stop();

	private:
	    bool in_loop_;
    };

    class dialog: public ::dialog
    {
	typedef std::map<std::string,QWidget*> widget_map;
	typedef std::map<std::string,long> command_map;
	typedef std::map<std::string,::dialog::widget_type> type_map;
	typedef std::stack<QBoxLayout*> layout_stack;
	typedef std::map<std::string,std::string> variable_map;

	public:
	    dialog();

	    void add_button(const std::string&, const std::string&, long);
	    void add_label(const std::string&,const std::string&);
	    void add_listbox(const std::string&,const std::string&,long);
	    void add_textbox(const std::string&,const std::string&,long);
	    void add_variable(const std::string&,const std::string&);
	    
	    void begin_hbox();
	    void end_hbox();
	    void begin_vbox();
	    void end_vbox();

	    void set_attr(const std::string&,const std::string&,const std::string&);
	    std::string get_attr(const std::string&,const std::string&);

	    long run();

	private:
	    dialog_widget	main_widget_;/**< reference to main widget */
	    layout_stack	lstack_; /**< current active layout */
	
	    widget_map	widgets_; /**< mapping: widget id --> pointer to widget */
	    command_map	commands_; /**< mapping: widget id --> pointer to command */
	    type_map	types_; /**< mapping: widget id --> type of widget */
	    variable_map vars_; /**< mapping: variable id --> variable value */
    };

}

#endif // QT_DIALOG_HH
